@@ -9,7 +9,7 @@ from mlagents.trainers.model_saver.torch_model_saver import TorchModelSaver
 from mlagents.trainers.model_saver.torch_model_saver import DEFAULT_CHECKPOINT_NAME
 
 from animo_trainer.animo_training_session import AnimoTrainingSession
-from TransformsAI.Animo.Data.Utility import Serializer
+from TransformsAI.Animo.Data import Serializer
 
 
 class AnimoTorchModelSaver(TorchModelSaver):
