@@ -1,12 +1,12 @@
-import pathlib, os
+import pathlib, os, shutil, time, yaml
 import numpy as np
 from typing import Dict, Optional
 
 from mlagents.trainers.cli_utils import parser as ml_agents_parser
 from mlagents.trainers.settings import RunOptions, TrainerSettings
 
-from TransformsAI.Animo.Data.Models import AgentData, LevelData
-from TransformsAI.Animo.Data.Utility import AgentCheckpointAccumulator, Serializer
+from TransformsAI.Animo.Data import AgentData, LevelData
+from TransformsAI.Animo.Data import AgentCheckpointAccumulator, Serializer
 
 
 INPUT_FOLDER_NAME = "Input"
@@ -34,7 +34,7 @@ class AnimoTrainingSession:
         else:
             self.should_check_daemon = False
 
-        ## Folders
+        ## Input folders
         training_session_full_folder_path = pathlib.Path(folder_path)
         if not os.path.isdir(training_session_full_folder_path):
             raise ValueError("Training session folder not found")
@@ -47,12 +47,12 @@ class AnimoTrainingSession:
         if not os.path.isdir(input_agents_full_path):
             raise ValueError("Input agents folder not found")
 
-        ## Config
-        config_full_path = os.path.join(input_folder_full_path, CONFIG_FILE_NAME)
-        if not os.path.isfile(config_full_path):
+        # Config
+        input_config_full_path = os.path.join(input_folder_full_path, CONFIG_FILE_NAME)
+        if not os.path.isfile(input_config_full_path):
             raise ValueError("Config.yaml not found")
 
-        args = ml_agents_parser.parse_args([config_full_path])
+        args = ml_agents_parser.parse_args([input_config_full_path])
         self.run_options = RunOptions.from_argparse(args)
 
         if self.run_options.env_settings.seed == -1:
@@ -62,21 +62,24 @@ class AnimoTrainingSession:
         self.run_options.checkpoint_settings.run_id = self.id
         self.run_options.checkpoint_settings.results_dir = str(os.path.join(training_session_full_folder_path, OUTPUT_FOLDER_NAME, AGENTS_FOLDER_NAME))
 
-        ##TODO: check if path exists and offset the name or smth to allow for running multiple sessions with the same params
-
-        ## Level
-        level_full_path = os.path.join(input_folder_full_path, LEVEL_FILE_NAME)
-        if not os.path.isfile(level_full_path):
+        # Level
+        input_level_full_path = os.path.join(input_folder_full_path, LEVEL_FILE_NAME)
+        if not os.path.isfile(input_level_full_path):
             raise ValueError("Level.json not found")
 
         json_level = None
-        with open(level_full_path, mode='r') as file:
+        with open(input_level_full_path, mode='r') as file:
             json_level = file.read()
         level_data: LevelData = Serializer.FromJson[LevelData](json_level)
         self.level_data = level_data
 
-        #Agents
-        for character in level_data.SavedGrid.Characters:
+        # Agents
+        characters = level_data.SavedGrid.Characters
+
+        if characters.Count < 1:
+            raise ValueError("Level.json does not contain any characters")
+
+        for character in characters:
             character_id: str = str(character.CharacterId)
 
             agent_full_path = os.path.join(input_agents_full_path, character_id)
@@ -101,3 +104,22 @@ class AnimoTrainingSession:
                 trainer_settings.init_path = agent_checkpoint_full_path
 
             self.agent_output_paths[character_id] = os.path.join(training_session_full_folder_path, OUTPUT_FOLDER_NAME, AGENTS_FOLDER_NAME, character_id)
+
+        #Output folders and files
+        output_folder_full_path = os.path.join(training_session_full_folder_path, OUTPUT_FOLDER_NAME)
+        if os.path.isdir(output_folder_full_path):
+            timestamp = int(time.time_ns()/1000)
+            new_folder_name = OUTPUT_FOLDER_NAME + "_" + str(timestamp)
+            new_folder_full_path = os.path.join(training_session_full_folder_path, new_folder_name)
+            print(f"Found existing output folder. Renaming it to {new_folder_name}")
+            os.rename(output_folder_full_path, new_folder_full_path)
+
+        os.mkdir(output_folder_full_path)
+
+        output_config_full_path = os.path.join(output_folder_full_path, CONFIG_FILE_NAME)
+        with open(output_config_full_path, 'x') as f:
+            yaml.dump(self.run_options.as_dict(), f, sort_keys=False)
+
+        output_level_full_path = os.path.join(output_folder_full_path, LEVEL_FILE_NAME)
+        shutil.copy(input_level_full_path, output_level_full_path)
+
